package cn.taiqing.dbExcel;

import java.io.FileOutputStream;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;

import cn.taiqing.dsutils.DataSourceUtils;

/**
 * 将数据库中的任意表到处到Excel
 * 
 * @author 泰庆
 * 
 */
public class DB2ExcelTemplate {

	/**
	 * 要导出的数据库名称
	 */
	private String dbName = null;
	/**
	 * 要导出的表名数组
	 */
	private List<String> tableNames = new ArrayList<String>();
	/**
	 * 要导出的Excel路径
	 */
	private String filepath = null;

	private Connection con;

	public DB2ExcelTemplate() {
		this.con = DataSourceUtils.getConnection();
	}

	public void run() {
		if (null != dbName) {
			try {
				//1.链接数据库，并读取信息
				Statement st = con.createStatement();
				DatabaseMetaData dbmd = con.getMetaData();
				ResultSet rs = dbmd.getTables(dbName, dbName, null, new String[]{"TABLE"});
				if(0 == tableNames.size()){
					while(rs.next()){
						String tableName = rs.getString("TABLE_NAME");
						tableNames.add(tableName);
					}
				}
				//2.将数据库信息导出到Excel中
				HSSFWorkbook book = new HSSFWorkbook();
				for(String tableName : tableNames){
					HSSFSheet sheet = book.createSheet(tableName);
					String sql = "SELECT * FROM " + dbName +"."+tableName;
					rs = st.executeQuery(sql);
					ResultSetMetaData rsmd = rs.getMetaData();
					int cols = rsmd.getColumnCount();
					HSSFRow row = sheet.createRow(0);
					for(int i = 0; i < cols ; i++){
						String column = rsmd.getColumnName(i+1);
						HSSFCell cell = row.createCell(i);
						cell.setCellValue(column);
					}
					int index = 1;
					while(rs.next()){
						row = sheet.createRow(index++);
						for(int i = 0; i < cols ; i++){
							String value = rs.getString(i+1);
							HSSFCell cell = row.createCell(i);
							cell.setCellValue(value);
						}
					}
				}
				con.close();
				if(null == filepath){
					String path = DB2ExcelTemplate.class.getClassLoader().getResource("jdbc.properties").getPath();
					filepath = path.substring(0, path.length()-15)+dbName+".xls";
				}
				book.write(new FileOutputStream(filepath));
				System.err.println("文件成功保存在：" + filepath);
			} catch (Exception e) {
				throw new RuntimeException(e.getMessage(), e);
			}
		} else {
			System.err.println("未指定数据库名称，打印取消");
		}
	}

	public void setDbname(String dbname) {
		this.dbName = dbname;
	}

	public void setTableNames(List<String> tableNames) {
		this.tableNames = tableNames;
	}

	public DB2ExcelTemplate addTableName(String tableName) {
		this.tableNames.add(tableName);
		return this;
	}

	public void setFilepath(String filepath) {
		this.filepath = filepath;
	}
}
