package cn.taiqing.dbpool;

import java.io.PrintWriter;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.SQLFeatureNotSupportedException;
import java.util.LinkedList;
import java.util.logging.Logger;

import javax.sql.DataSource;

import cn.taiqing.utils.PropertiesUtil;

/**
 * 动态连接池
 * 使用代理实现
 * @author 泰庆
 */
public class EasyDBProxyPool implements DataSource,IEasyDBPool{
	
	@Override
	public Connection getConn() throws SQLException{
		return this.getConnection();
	}
	
	// 1.声明连接池
	private LinkedList<Connection> pool = new LinkedList<Connection>();
	public EasyDBProxyPool(){
		// 2.获取配置信息
		try {
			Class.forName(PropertiesUtil.DRIVER);
			for (int i = 0; i < PropertiesUtil.POOL_SIZE; i++) {
				final Connection con = DriverManager.getConnection(
						PropertiesUtil.URL, PropertiesUtil.NAME,
						PropertiesUtil.PWD);
				// 3.动态代理close方法
				Object connProxyed = Proxy.newProxyInstance(
						EasyDBProxyPool.class.getClassLoader(),
						new Class[] { Connection.class },
						new InvocationHandler() {
							@Override
							public Object invoke(Object proxy, Method method,
									Object[] args) throws Throwable {
								String methodName = method.getName();
								if ("close".equals(methodName)) {
									synchronized (pool) {
										pool.addLast((Connection) proxy);
										pool.notify();
										return null;
									}
								}
								return method.invoke(con, args);
							}
						});
				// 4.添加链接到连接池中
				pool.add((Connection) connProxyed);
			}
		} catch (Exception e) {
			throw new RuntimeException(e.getMessage(), e);
		}
	}

	// 5.对外提供getConnection方法
	@Override
	public Connection getConnection() throws SQLException {
		synchronized(pool){
			if(0 == pool.size()){
				try{
					pool.wait();
				}catch(Exception e){
					throw new RuntimeException(e.getMessage(),e);
				}
				return getConnection();
			}
		}
		return pool.removeFirst();
	}

	@Override
	public PrintWriter getLogWriter() throws SQLException {
		return null;
	}

	@Override
	public void setLogWriter(PrintWriter out) throws SQLException {

	}

	@Override
	public void setLoginTimeout(int seconds) throws SQLException {

	}

	@Override
	public int getLoginTimeout() throws SQLException {
		return 0;
	}

	@Override
	public Logger getParentLogger() throws SQLFeatureNotSupportedException {
		return null;
	}

	@Override
	public <T> T unwrap(Class<T> iface) throws SQLException {
		return null;
	}

	@Override
	public boolean isWrapperFor(Class<?> iface) throws SQLException {
		return false;
	}

	@Override
	public Connection getConnection(String username, String password)
			throws SQLException {
		return null;
	}

}
